Attribute VB_Name = "mMap"
Option Explicit

'Types
    Public Type tMap
        W As Long 'W stands for width
        H As Long 'H stands for height
        
        'Maps
        Floor() As Long
        Roof() As Long
        Units() As Long
    End Type

'Variables
    Public Map As tMap

Public Sub CreateMap(iW As Long, iH As Long)
    Dim A As Long
    
    With Map
        'Get size
        .W = iW
        .H = iH
        
        'Create maps
        ReDim .Floor(.W * .H)
        ReDim .Roof(.W * .H)
        ReDim .Units(.W * .H)
    End With
End Sub

Public Sub Reset()
    'Reset map
    Map.W = 0
    Map.H = 0
    
    Erase Map.Floor
    Erase Map.Roof
End Sub

Public Sub DrawFloor()
    Dim X As Long
    Dim Y As Long
    
    Dim Index As Long
    Dim TileIndex As Long
    
    Dim TempX As Single
    Dim TempY As Single
    
    With Map
        'Get map position
        TempX = (Camera.X / TileSize)
        TempY = (Camera.Y / TileSize)
        
        'Calculate index of this positon
        Index = Int(TempY) * .W + Int(TempX)
        
        'Calculate offset
        TempX = TempX - Int(TempX)
        TempY = TempY - Int(TempY)
        
        'Draw the map
        For Y = 0 To Camera.TilesY
            For X = 0 To Camera.TilesX
                'Check index
                If Index > (Map.W * .H) Then: Exit Sub
                
                'Get tile index
                TileIndex = .Floor(Index)
                
                'Draw tile
                BitBlt Camera.FrontDC, _
                    (X - TempX) * TileSize, (Y - TempY) * TileSize, _
                    TileSize, TileSize, _
                    Bitmap(Tile(TileIndex).Bitmap).Picture.DC, _
                    Tile(TileIndex).ActFrame * TileSize, 0, _
                    vbSrcCopy
                
                'Next tile
                Index = Index + 1
            Next
            
            'Get next line start
            Index = Index + .W - Camera.TilesX - 1
        Next
    End With
End Sub
Public Sub DrawUnits()
    Dim X As Long
    Dim Y As Long
    
    Dim Index As Long
    Dim UnitIndex As Long
    
    Dim OffsetX As Long
    Dim OffsetY As Long
    
    Dim DrawX As Long
    Dim DrawY As Long
    
    'Calculate start index
    Index = Int((Camera.Y / TileSize)) * Map.W + Int((Camera.X / TileSize))
    
    'Draw the map
    For Y = 0 To Camera.TilesY
        For X = 0 To Camera.TilesX
            'Check index
            If Index > (Map.W * Map.H) Then: Exit Sub
            
            'Get unit index
            UnitIndex = Map.Units(Index)
            
            If UnitIndex > 0 Then
                With Unit(UnitIndex)
                    'Get offset position (bottom-center)
                    OffsetX = .W / 2 - TileSize / 2
                    OffsetY = .H - TileSize / 2
                    
                    'Get final draw position
                    DrawX = (.X - OffsetX) - Camera.X
                    DrawY = (.Y - OffsetY) - Camera.Y
                    
                    'Draw mask
                    BitBlt Camera.FrontDC, _
                        DrawX, DrawY, _
                        .W, .H, _
                        Bitmap(.Bitmap).Mask.DC, _
                        (.W * .ActFrame), (.H * .Direction), _
                        vbSrcPaint
                    
                    'Draw picture
                    BitBlt Camera.FrontDC, _
                        DrawX, DrawY, _
                        .W, .H, _
                        Bitmap(.Bitmap).Picture.DC, _
                        (.W * .ActFrame), (.H * .Direction), _
                        vbSrcAnd
                End With
            End If
            
            'Next tile
            Index = Index + 1
        Next
        
        'Get next line start
        Index = Index + Map.W - Camera.TilesX - 1
    Next
End Sub
Public Sub DrawRoof()
    Dim X As Long
    Dim Y As Long
    
    Dim Index As Long
    Dim TileIndex As Long
    
    Dim TempX As Single
    Dim TempY As Single
    
    With Map
        'Get map position
        TempX = (Camera.X / TileSize)
        TempY = (Camera.Y / TileSize)
        
        'Calculate index of this positon
        Index = Int(TempY) * .W + Int(TempX)
        
        'Calculate offset
        TempX = TempX - Int(TempX)
        TempY = TempY - Int(TempY)
        
        'Draw the map
        For Y = 0 To Camera.TilesY
            For X = 0 To Camera.TilesX
                'Check index
                If Index > (.W * .H) Then: Exit Sub
                
                'Get tile index
                TileIndex = .Roof(Index)
                
                If TileIndex > 0 Then
                    'Draw tile mask
                    BitBlt Camera.FrontDC, _
                        (X - TempX) * TileSize, (Y - TempY) * TileSize, _
                        TileSize, TileSize, _
                        Bitmap(Tile(TileIndex).Bitmap).Mask.DC, _
                        Tile(TileIndex).ActFrame * TileSize, 0, _
                        vbSrcPaint 'logical AND
                    
                    'Draw roof tile
                    BitBlt Camera.FrontDC, _
                        (X - TempX) * TileSize, (Y - TempY) * TileSize, _
                        TileSize, TileSize, _
                        Bitmap(Tile(TileIndex).Bitmap).Picture.DC, _
                        Tile(TileIndex).ActFrame * TileSize, 0, _
                        vbSrcAnd 'logical OR
                End If
                
                'Next tile
                Index = Index + 1
            Next
            
            'Get next line start
            Index = Index + .W - Camera.TilesX - 1
        Next
    End With
End Sub

Public Function SaveData(iFileName As String) As Boolean
On Error GoTo 1

    Dim FileNumber As Integer
    
    'Open the specified file
    FileNumber = FreeFile
    Open iFileName For Binary As FileNumber
        'Write data
        WriteData FileNumber
    Close FileNumber
    
    'We're done
    SaveData = True
    Exit Function

1  'Catched an error
    SaveData = False
End Function

Public Function LoadData(iFileName As String) As Boolean
On Error GoTo 1

    Dim FileNumber As Integer
    
    'Check existance
    If Dir(iFileName, vbNormal) = "" Then: GoTo 1
    
    'Open the specified file
    FileNumber = FreeFile
    Open iFileName For Binary As FileNumber
        'Read data
        ReadData FileNumber
    Close FileNumber
    
    'We're done
    LoadData = True
    Exit Function

1  'Catched an error
    LoadData = False
End Function

Public Sub SetFloorTile(iX As Long, iY As Long, Index As Long)
    'Set tile at the specified 0-based position
    Map.Floor(iY * Map.W + iX) = Index
End Sub

Public Sub SetRoofTile(iX As Long, iY As Long, Index As Long)
    'Set tile at the specified 0-based position
    Map.Roof(iY * Map.W + iX) = Index
End Sub
Public Sub WriteData(iFileNumber As Integer)
    'Write tile set
    mTiles.WriteData iFileNumber
    
    With Map
        'Write map size
        Put iFileNumber, , .W
        Put iFileNumber, , .H
        
        'Write map data
        Put iFileNumber, , .Floor
        Put iFileNumber, , .Roof
    End With
End Sub

Public Sub ReadData(iFileNumber As Integer)
    'Read tile set
    mTiles.ReadData iFileNumber
        
    With Map
        'Read m size
        Get iFileNumber, , .W
        Get iFileNumber, , .H
        
        'Allocate memory
        CreateMap .W, .H
        
        'Read map data
        Get iFileNumber, , .Floor
        Get iFileNumber, , .Roof
    End With
End Sub



