Attribute VB_Name = "mUnits"
Option Explicit

'Types
    Public Type tUnit
        'Position (in pixel)
        X As Long
        Y As Long
        
        'Current map position
        MapIndex As Long
        Moved As Boolean
        
        'Unit size
        W As Long
        H As Long
        
        'Animation setup
        FrameCount As Integer
        FrameDelay As Long
        
        'Animation data
        Animated As Boolean
        ActFrame As Integer
        NextTick As Long
        
        'Display
        Bitmap As Long
        Direction As Long
    End Type

'Variables
    Public UnitCount As Long
    Public Unit() As tUnit

Public Sub AddUnit(iPictureFile As String, iW As Long, iH As Long, iFrameCount As Long, iFrameDelay As Long)
    'Allocate memory
    UnitCount = UnitCount + 1
    ReDim Preserve Unit(UnitCount)
    
    With Unit(UnitCount)
        'Load bitmap
        .Bitmap = LoadBitmap(SourcePath & iPictureFile, True)
        
        'Setup size
        .W = iW
        .H = iH
        
        'Setup animation info
        .FrameCount = iFrameCount
        .FrameDelay = iFrameDelay
        
        'Reset animation
        .ActFrame = 0
        .NextTick = 0
        
        'Reset map index
        .MapIndex = -1
    End With
End Sub

Public Sub Animate()
    Dim A As Long
    Dim Temp As Long
    
    'Get current tick
    Temp = GetTickCount
    
    'Check all units
    For A = 0 To UnitCount
        With Unit(A)
            'Check timing
            If .NextTick < Temp Then
                'Check if animated
                If .Animated Then
                    'Increase frame number
                    .ActFrame = .ActFrame + 1
                    
                    'Reset animation if last frame reached
                    If .ActFrame > .FrameCount Then: .ActFrame = 0
                    
                    'Set next tick
                    .NextTick = Temp + .FrameDelay
                
                Else
                    'Reset animation
                    .ActFrame = 0
                End If
            End If
        End With
    Next
End Sub
Public Sub MoveRandomly()
    Dim A As Long
    
    Dim MoveX As Long
    Dim MoveY As Long
    
    For A = 2 To UnitCount
        'Reset
        MoveX = 0
        MoveY = 0
        
        With Unit(A)
            'Set random direction
            If 100 * Rnd < 10 Then: .Direction = CLng(3 * Rnd)
            
            'Get movement
            Select Case .Direction
                Case 0: MoveY = -1 'Up
                Case 1: MoveX = 1 'Right
                Case 2: MoveY = 1 'Down
                Case 3: MoveX = -1 'Left
            End Select
            
            'Set new position
            MoveUnit A, (.X + MoveX), (.Y + MoveY)
        End With
    Next
End Sub

Public Sub MoveUnit(Index As Long, NewX As Long, NewY As Long)
    Dim TempIndex As Long
    
    'Check boundaries
    If NewX < 0 Then: NewX = 0
    If NewY < 0 Then: NewY = 0
    If NewX > ((Map.W - 1) * TileSize) Then: NewX = ((Map.W - 1) * TileSize)
    If NewY > ((Map.H - 1) * TileSize) Then: NewY = ((Map.H - 1) * TileSize)
    
    'Get new map index
    TempIndex = (CLng(NewY / TileSize) * Map.W) + CLng(NewX / TileSize)
    
    With Unit(Index)
        'Reset movement indicator
        .Moved = False
        
        If Not TempIndex = .MapIndex Then
            'Unit-to-map collision check
            If Not Tile(Map.Floor(TempIndex)).Walkable Then: Exit Sub
            
            'Unit-to-unit collision check
            If Map.Units(TempIndex) > 0 Then: Exit Sub
        End If
        
        'Unregister from map
        If .MapIndex > -1 Then: Map.Units(.MapIndex) = 0
        
        'Apply new position
        .X = NewX
        .Y = NewY
        .MapIndex = TempIndex
        
        'We moved
        .Moved = True
        
        'Register to map
        Map.Units(.MapIndex) = Index
    End With
End Sub

Public Sub Reset()
    'Reset units
    UnitCount = -1
    Erase Unit
End Sub

Public Sub UpdateAnimations()
    Dim A As Long
    
    'Frequently update
    For A = 0 To UnitCount
        With Unit(A)
            'Animate if moved
            .Animated = .Moved
            .Moved = False
        End With
    Next
End Sub


